from collective.collectionfilter import _
from collective.collectionfilter.filteritems import CollectionishCollection
from collective.collectionfilter.interfaces import ICollectionish
from plone import api
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.layoutbehavior import ILayoutBehaviorAdaptable
from plone.app.contenttypes.behaviors.collection import ICollection
from plone.tiles.tile import PersistentTile
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import queryAdapter
from zope.interface import implementer

import re


# tile names actijg collectionish
COLLECTIONISH_TARGETS = [
    "plone.app.standardtiles.contentlisting",
]


class DictDataWrapper:
    def __init__(self, data):
        self.data = data

    def __getattr__(self, name):
        try:
            return self.data[name]
        except KeyError:
            raise AttributeError


class BaseFilterTile(PersistentTile):
    def available(self):
        # do not render when page is ajax loaded
        return "ajax_load" not in self.top_request and self.is_available

    @property
    def edit_url(self):
        if not api.user.has_permission("cmf.ModifyPortalContent", obj=self.context):
            return None
        return self.url.replace("@@", "@@edit-tile/")

    @property
    def settings(self):
        return DictDataWrapper(self.data)

    @property
    def filter_id(self):
        return self.id

    @property
    def reload_url(self):
        return self.url


def findall_tiles(context, spec):
    if not isinstance(spec, list):
        spec = [spec]
    request = context.REQUEST
    la = ILayoutAware(context)
    layout = (
        la.customContentLayout
        if la.customContentLayout is not None
        else la.contentLayout if la.contentLayout is not None else la.content
    )
    if layout is None:
        return []
    possible_urls = re.findall(r"(@@[\w\.]+/\w+)", layout)
    urls = []
    for name in spec:
        urls += [url for url in possible_urls if url.startswith(f"@@{name}")]
    # TODO: maybe better to get tile data? using ITileDataManager(id)?
    our_tile = request.response.headers.get("x-tile-url")
    tiles = [context.unrestrictedTraverse(str(url)) for url in urls]
    # mosaic get confused if we are doing this while creating a filter tile
    if request.response.headers.get("x-tile-url"):
        if our_tile:
            request.response.headers["x-tile-url"] = our_tile
        else:
            del request.response.headers["x-tile-url"]
    return tiles


@implementer(ICollectionish)
@adapter(ILayoutBehaviorAdaptable)
class CollectionishLayout(CollectionishCollection):
    """Provide interface for either objects with contentlisting tiles or collections or both"""

    tile = None
    collection = None

    def __init__(self, context):
        """Adapt either collections or contentlisting tile. The name is sorted content selector"""
        self.context = context

        # self.context might not be adaptable
        # make sure you select the right content with
        # selectContent(selector) after initialization
        try:
            self.collection = ICollection(self.context)
        except TypeError:
            self.collection = None

    def selectContent(self, selector=None):
        """Pick tile that selector will match, otherwise pick first one.
        Return None if no listing tile or collection is suitable, else return this adapter.
        """

        if selector is None:
            selector = ""
        self.tile = None
        tiles = findall_tiles(self.context, COLLECTIONISH_TARGETS)
        for tile in tiles:
            tile.update()
            tile_classes = getattr(tile, "tile_class", "").split() + [""]
            # First tile that matches all the selector classes
            if all([_class in tile_classes for _class in selector.split(".")]):
                self.tile = tile
                break
        if tiles and self.tile is None:
            # TODO: what if the class is inside a special template? Just pick first?
            # none of the selectors worked. Just pick any and hope it works?
            self.tile = tile
        if self.tile is not None:
            # we have to set self.collection always to self.tile to get the
            # correct query
            self.collection = self.tile
            return self
        if self.collection is not None:
            # this context is a Collection ILayoutAware enabled but there were
            # no COLLECTIONISH_TARGETS tiles found. Likely because "layout_view"
            # is not enabled here.
            return self

    @property
    def sort_reversed(self):
        if self.tile is not None:
            return self.sort_order == "reverse"
        return self.collection.sort_reversed

    @property
    def content_selector(self):
        """will return None if no tile or collection found"""
        if self.collection is None:
            return
        if self.tile is None:
            return super().content_selector
        classes = ["contentlisting-tile"]
        if getattr(self.tile, "tile_class", ""):
            classes += self.tile.tile_class.split()
        return "." + ".".join(classes)

    def results(self, custom_query, request_params):
        """Search results"""
        if self.tile is None:
            return super().results(custom_query, request_params)

        builder = getMultiAdapter(
            (self.context, self.context.REQUEST), name="querybuilderresults"
        )

        # Include query parameters from request if not set to ignore
        contentFilter = {}
        if not getattr(self.tile, "ignore_request_params", False):
            contentFilter = dict(self.context.REQUEST.get("contentFilter", {}))

        # TODO: handle events extra params

        return builder(
            query=self.query,
            sort_on=self.sort_on or "getObjPositionInParent",
            sort_order=self.sort_order,
            limit=self.limit,
            batch=False,
            brains=True,
            custom_query=custom_query if custom_query is not None else contentFilter,
        )


def validateFilterTileModify(tile, event):
    # TODO: is ok in the acquisition path?
    target = tile.collection
    if target is not None:
        target = queryAdapter(target.getObject(), ICollectionish)
    if target is not None:
        target = target.selectContent("")
    if target is None or target.content_selector is None:
        api.portal.show_message(
            _(
                "You will need to add a Content Listing tile or target a collection to make Filters work"
            ),
            request=tile.context.REQUEST,
            type="warning",
        )
        return False
    return True


def validateFilterMosaicModify(context, event):
    # search the layout for any filters and then see if they have a matching listing
    tiles = findall_tiles(context, "collective.collectionfilter.tiles.")
    for tile in tiles:
        if not validateFilterTileModify(tile, event):
            break



"""Setup tests for this package."""

from collective.collectionfilter.testing import (  # noqa
    COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.CMFCore.utils import getToolByName

import unittest


no_get_installer = False

try:
    from Products.CMFPlone.utils import get_installer
except Exception:
    # Quick shim for 5.1 api change

    class get_installer:
        def __init__(self, portal, request):
            self.installer = getToolByName(portal, "portal_quickinstaller")

        def is_product_installed(self, name):
            return self.installer.isProductInstalled(name)

        def uninstall_product(self, name):
            return self.installer.uninstallProducts([name])


class TestSetup(unittest.TestCase):
    """Test that collective.collectionfilter is properly installed."""

    layer = COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.installer = get_installer(self.portal, self.layer["request"])

    def test_product_installed(self):
        """Test if collective.collectionfilter is installed."""
        self.assertTrue(
            self.installer.is_product_installed("collective.collectionfilter")
        )

    def test_browserlayer(self):
        """Test that ICollectionFilterBrowserLayer is registered."""
        from collective.collectionfilter.interfaces import ICollectionFilterBrowserLayer
        from plone.browserlayer import utils

        self.assertIn(ICollectionFilterBrowserLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):
    layer = COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.installer = get_installer(self.portal, self.layer["request"])
        roles_before = api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer.uninstall_product("collective.collectionfilter")
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        """Test if collective.collectionfilter is cleanly uninstalled."""
        self.assertFalse(
            self.installer.is_product_installed("collective.collectionfilter")
        )

    def test_browserlayer_removed(self):
        """Test that ICollectionFilterBrowserLayer is removed."""
        from collective.collectionfilter.interfaces import ICollectionFilterBrowserLayer
        from plone.browserlayer import utils

        self.assertNotIn(ICollectionFilterBrowserLayer, utils.registered_layers())



from collective.collectionfilter.baseviews import BaseResetFilterView
from collective.collectionfilter.interfaces import ICollectionResetFilterSchema
from collective.collectionfilter.tiles import BaseFilterTile
from plone.supermodel.model import Schema
from zope.interface import implementer


class IResetFilterTile(Schema, ICollectionResetFilterSchema):
    pass


@implementer(IResetFilterTile)
class ResetFilterTile(BaseFilterTile, BaseResetFilterView):
    """Reset Filter Tile"""



from collective.collectionfilter import _
from collective.collectionfilter.baseviews import BaseFilterView
from collective.collectionfilter.interfaces import ICollectionFilterSchema
from collective.collectionfilter.portlets import BasePortletRenderer
from collective.collectionfilter.vocabularies import DEFAULT_FILTER_TYPE
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implementer


class ICollectionFilterPortlet(ICollectionFilterSchema, IPortletDataProvider):
    """Portlet interface based on ICollectionFilterSchema"""


@implementer(ICollectionFilterPortlet)
class Assignment(base.Assignment):
    header = ""
    target_collection = None
    group_by = ""
    show_count = False
    cache_enabled = True
    filter_type = DEFAULT_FILTER_TYPE
    input_type = "links"
    narrow_down = False
    view_name = None
    content_selector = "#content-core"
    hide_if_empty = False
    reverse = False
    # list_scaling = None

    def __init__(
        self,
        header="",
        target_collection=None,
        group_by="",
        show_count=False,
        cache_enabled=True,
        filter_type=DEFAULT_FILTER_TYPE,
        input_type="links",
        narrow_down=False,
        view_name=None,
        content_selector="#content-core",
        hide_if_empty=False,
        reverse=False,
        # list_scaling=None
    ):
        self.header = header
        self.target_collection = target_collection
        self.group_by = group_by
        self.show_count = show_count
        self.cache_enabled = cache_enabled
        self.filter_type = filter_type
        self.input_type = input_type
        self.narrow_down = narrow_down
        self.view_name = view_name
        self.content_selector = content_selector
        self.hide_if_empty = hide_if_empty
        self.reverse = reverse
        # self.list_scaling = list_scaling

    @property
    def portlet_id(self):
        """Return the portlet assignment's unique object id."""
        return id(self)

    @property
    def title(self):
        if self.header:
            return self.header
        else:
            return _("Collection Filter")


class Renderer(BasePortletRenderer, BaseFilterView):
    render = ViewPageTemplateFile("collectionfilter.pt")


class AddForm(base.AddForm):
    schema = ICollectionFilterPortlet
    label = _("Add Collection Filter Portlet")
    description = _(
        "This portlet shows grouped criteria of collection results and "
        "allows filtering of collection results."
    )

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    schema = ICollectionFilterPortlet
    label = _("Edit Collection Filter Portlet")
    description = _(
        "This portlet shows grouped criteria of collection results and "
        "allows filtering of collection results."
    )



from collective.collectionfilter.testing import (
    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_DISABLED,
)
from collective.collectionfilter.testing import (
    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_ENABLED,
)
from collective.collectionfilter.testing import (
    COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_TILES,
)
from plone.app.testing import ROBOT_TEST_LEVEL
from plone.testing import layered

import robotsuite
import unittest


def test_suite():
    suite = unittest.TestSuite()
    test_layer_mapping = (
        (
            COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_DISABLED,
            ("debug", "filterportlets", "sortingportlets"),
        ),
        (
            COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_ENABLED,
            ("debug", "filterportlets", "sortingportlets"),
        ),
        (
            COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_TILES,
            ("tilesonly",),
        ),
    )
    for layer, test_files in test_layer_mapping:
        for robot_test in test_files:
            robottestsuite = robotsuite.RobotTestSuite(f"robot/test_{robot_test}.robot")
            robottestsuite.level = ROBOT_TEST_LEVEL
            suite.addTests([layered(robottestsuite, layer=layer)])

    return suite



from collective.collectionfilter import _
from collective.collectionfilter.baseviews import BaseSearchView
from collective.collectionfilter.interfaces import ICollectionSearchSchema
from collective.collectionfilter.portlets import BasePortletRenderer
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implementer


class ICollectionSearchPortlet(ICollectionSearchSchema, IPortletDataProvider):
    """Portlet interface based on ICollectionSearchSchema"""


@implementer(ICollectionSearchPortlet)
class Assignment(base.Assignment):
    header = ""
    target_collection = None
    view_name = None
    content_selector = "#content-core"

    def __init__(
        self,
        header="",
        target_collection=None,
        view_name=None,
        content_selector="#content-core",
    ):
        self.header = header
        self.target_collection = target_collection
        self.view_name = view_name
        self.content_selector = content_selector

    @property
    def title(self):
        if self.header:
            return self.header
        else:
            return _("Collection Search")


class Renderer(BasePortletRenderer, BaseSearchView):
    render = ViewPageTemplateFile("collectionsearch.pt")


class AddForm(base.AddForm):
    schema = ICollectionSearchPortlet
    label = _("Add Collection Search Portlet")
    description = _("This portlet allows fulltext search in collection results.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    schema = ICollectionSearchPortlet
    label = _("Edit Collection Search Portlet")
    description = _("This portlet allows fulltext search in collection results.")



from collective.collectionfilter import _
from collective.collectionfilter.baseviews import BaseMapsView
from collective.collectionfilter.interfaces import ICollectionMapsSchema
from collective.collectionfilter.portlets import BasePortletRenderer
from plone.app.portlets.portlets import base
from plone.formwidget.geolocation.vocabularies import default_map_layer
from plone.formwidget.geolocation.vocabularies import default_map_layers
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implementer


class ICollectionMapsPortlet(ICollectionMapsSchema, IPortletDataProvider):
    """Portlet interface based on ICollectionMapsSchema"""


@implementer(ICollectionMapsPortlet)
class Assignment(base.Assignment):
    header = ""
    target_collection = None
    view_name = None
    content_selector = "#content-core"
    narrow_down = False
    default_map_layer = default_map_layer
    map_layers = default_map_layers
    geojson_properties_limit = 500

    def __init__(
        self,
        header="",
        target_collection=None,
        view_name=None,
        content_selector="#content-core",
        narrow_down=False,
        default_map_layer=default_map_layer,
        map_layers=default_map_layers,
        geojson_properties_limit=500,
    ):
        self.header = header
        self.target_collection = target_collection
        self.view_name = view_name
        self.content_selector = content_selector
        self.narrow_down = narrow_down
        self.default_map_layer = default_map_layer
        self.map_layers = map_layers
        self.geojson_properties_limit = geojson_properties_limit

    @property
    def title(self):
        if self.header:
            return self.header
        else:
            return _("Collection Maps")


class Renderer(BasePortletRenderer, BaseMapsView):
    render = ViewPageTemplateFile("maps.pt")


class AddForm(base.AddForm):
    schema = ICollectionMapsPortlet
    label = _("Add Collection Maps Portlet")
    description = _("This portlet allows map filtering in collection results.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    schema = ICollectionMapsPortlet
    label = _("Edit Collection Maps Portlet")
    description = _("This portlet allows map filtering in collection results.")



from .filteritems import get_filter_items
from .filteritems import ICollectionish
from .interfaces import IGroupByCriteria
from .query import make_query
from .utils import base_query
from .utils import clean_query
from .utils import safe_decode
from .utils import safe_encode
from .utils import safe_iterable
from .vocabularies import TEXT_IDX
from Acquisition import aq_inner
from plone import api
from plone.api.portal import get_registry_record as getrec
from plone.app.uuid.utils import uuidToCatalogBrain
from plone.base.utils import get_top_request
from plone.base.utils import safe_text
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize import instance
from plone.uuid.interfaces import IUUID
from Products.Five import BrowserView
from urllib.parse import urlencode
from zope.component import getUtility
from zope.component import queryUtility
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory

import json


try:
    from collective.geolocationbehavior.interfaces import IGeoJSONProperties
    from plone.formwidget.geolocation.vocabularies import _

    HAS_GEOLOCATION = True
except ImportError:
    HAS_GEOLOCATION = False


class BaseView:
    """Abstract base filter view class."""

    _collection = None
    _top_request = None

    @property
    def settings(self):
        return self.data

    @property
    def available(self):
        return True

    @property
    def is_available(self):
        return True

    @property
    def filter_id(self):
        raise NotImplementedError

    @property
    def title(self):
        return getattr(self.settings, "header", getattr(self.collection, "Title", ""))

    @property
    def filterClassName(self):
        name = self.title and queryUtility(IIDNormalizer).normalize(self.title)
        return "filter" + name.capitalize() if name else ""

    @property
    def reload_url(self):
        raise NotImplementedError

    @property
    def top_request(self):
        if not self._top_request:
            self._top_request = get_top_request(self.request)
        return self._top_request

    @property
    def collection_uuid(self):
        if self.settings.target_collection:
            return self.settings.target_collection
        return IUUID(self.context)

    @property
    def collection(self):
        if not self._collection:
            self._collection = uuidToCatalogBrain(self.collection_uuid)
        return aq_inner(self._collection)

    @property
    def pat_options(self):
        return json.dumps(
            {
                "collectionUUID": self.collection_uuid,
                "reloadURL": self.reload_url,
                "ajaxLoad": self.ajax_load,
                "contentSelector": self.content_selector,
            }
        )

    @property
    def content_selector(self):
        if self.settings.content_selector:
            return self.settings.content_selector

        collectionish = (
            ICollectionish(self.collection.getObject()) if self.collection else None
        )
        if collectionish is not None:
            # select default content
            collectionish.selectContent()
        selector = collectionish.content_selector
        if collectionish is None or not selector:
            return "#content-core"
        else:
            return selector

    @property
    def ajax_load(self):
        values = api.portal.get_registry_record("plone.patternoptions")
        if "collectionfilter" in values:
            filterOptions = json.loads(values["collectionfilter"])
            if "ajaxLoad" in filterOptions:
                return filterOptions["ajaxLoad"]
        return True


class BaseFilterView(BaseView):
    @property
    def input_type(self):
        if self.settings.input_type == "links":
            return "link"
        if self.settings.filter_type == "single":
            if self.settings.input_type == "checkboxes_radiobuttons":
                return "radio"
            return "dropdown"
        return "checkbox"

    # results is called twice inside the template in view/available and view/results.  But its expensive so we cache it
    # but just the the lifetime of the view
    @instance.memoize
    def results(self):
        results = get_filter_items(
            target_collection=self.collection_uuid,
            group_by=self.settings.group_by,
            filter_type=self.settings.filter_type,
            narrow_down=self.settings.narrow_down,
            show_count=self.settings.show_count,
            view_name=self.settings.view_name,
            cache_enabled=self.settings.cache_enabled,
            request_params=self.top_request.form or {},
            content_selector=self.settings.content_selector,
            reverse=self.settings.reverse,
        )
        return results

    @property
    def is_available(self):
        if not self.settings.hide_if_empty:
            return True

        if self.settings.narrow_down:
            groupby_criteria = getUtility(IGroupByCriteria).groupby
            idx = groupby_criteria[self.settings.group_by]["index"]
            request_params = safe_decode(self.top_request.form)
            current_idx_value = safe_iterable(request_params.get(idx))
            if current_idx_value:
                return True

        results = self.results()
        return not (results is None or len(results) <= 2)  # 2 becayse we include "All"


class BaseSearchView(BaseView):
    @property
    def value(self):
        return safe_text(self.top_request.get(TEXT_IDX, ""))

    @property
    def action_url(self):
        return self.collection.getURL()

    @property
    def urlquery(self):
        urlquery = {}
        urlquery.update(self.top_request.form)

        ignore_params = [
            TEXT_IDX,
            "b_start",
            "b_size",
            "batch",
            "sort_on",
            "limit",
            "portlethash",
        ]
        urlquery = clean_query(urlquery, ignore_params)
        # add filter trigger if not already there
        if "collectionfilter" not in urlquery:
            urlquery["collectionfilter"] = "1"

        return ((k, vv) for k, v in urlquery.items() for vv in safe_iterable(v))

    @property
    def ajax_url(self):
        # Recursively transform all to unicode
        request_params = safe_decode(self.top_request.form)
        urlquery = base_query(request_params, extra_ignores=["SearchableText"])
        query_param = urlencode(safe_encode(urlquery), doseq=True)
        ajax_url = "/".join(
            [
                it
                for it in [
                    self.collection.getURL(),
                    self.settings.view_name,
                    "?" + query_param if query_param else None,
                ]
                if it
            ]
        )
        return ajax_url


class BaseSortOnView(BaseView):
    def results(self):
        collection = ICollectionish(self.collection.getObject()).selectContent(
            self.settings.content_selector
        )
        if collection is None:
            return
        curr_val = self.top_request.get("sort_on", collection.sort_on)
        curr_order = self.top_request.get(
            "sort_order", "descending" if collection.sort_reversed else "ascending"
        )
        new_order = "ascending"
        if curr_order is not None and curr_order == "ascending":
            new_order = "descending"
        sortable_indexes = getUtility(
            IVocabularyFactory, name="collective.collectionfilter.SortOnIndexes"
        )
        vocab = sortable_indexes(self.context)
        for idx in self.settings.sort_on:
            curr = curr_val == idx
            yield {
                "value": idx,
                "title": vocab.getTerm(idx).title,
                "new_order": new_order if curr else "ascending",
                "curr_order": curr_order if curr else "",
                "active": curr,
            }

    @property
    def ajax_url(self):
        # Recursively transform all to unicode
        request_params = safe_decode(self.top_request.form)
        urlquery = base_query(request_params, extra_ignores=["sort_on", "sort_order"])
        query_param = urlencode(safe_encode(urlquery), doseq=True)
        ajax_url = "/".join(
            [
                it
                for it in [
                    self.collection.getURL(),
                    self.settings.view_name,
                    "?" + query_param if query_param else None,
                ]
                if it
            ]
        )
        return ajax_url


if HAS_GEOLOCATION:

    class BaseMapsView(BaseView):
        @property
        def ajax_url(self):
            # Recursively transform all to unicode
            request_params = safe_decode(self.top_request.form)
            urlquery = base_query(
                request_params, extra_ignores=["latitude", "longitude"]
            )
            query_param = urlencode(safe_encode(urlquery), doseq=True)
            ajax_url = "/".join(
                [
                    it
                    for it in [
                        self.collection.getURL(),
                        self.settings.view_name,
                        "?" + query_param if query_param else None,
                    ]
                    if it
                ]
            )
            return ajax_url

        @property
        def geojson_ajax_url(self):
            # Recursively transform all to unicode
            request_params = safe_decode(self.top_request.form)
            urlquery = base_query(
                request_params, extra_ignores=["latitude", "longitude"]
            )
            query_param = urlencode(safe_encode(urlquery), doseq=True)
            geojson_ajax_url = "{}/@@geodata.json?geojson-limit={}{}".format(
                self.collection.getURL(),
                self.settings.geojson_properties_limit,
                "&" + query_param if query_param else "",
            )
            return geojson_ajax_url

        @property
        def map_configuration(self):
            config = {
                "fullscreencontrol": getrec("geolocation.fullscreen_control"),
                "locatecontrol": getrec("geolocation.locate_control"),
                "zoomcontrol": getrec("geolocation.zoom_control"),
                "minimap": getrec("geolocation.show_minimap"),
                "default_map_layer": self.settings.default_map_layer,
                "geojson_ajaxurl": self.geojson_ajax_url,
                "map_layers": [
                    {"title": translate(_(it), context=self.request), "id": it}
                    for it in self.settings.map_layers
                ],
            }
            return json.dumps(config)

    class GeoJSON(BrowserView):
        def __call__(self):
            """Return the geo location as GeoJSON string."""
            self.request.response.setHeader("Content-type", "application/json")
            features = []
            locations = self.locations
            count = len(locations)

            try:
                limit = int(self.request.get("geojson-limit"))
            except Exception:
                limit = 500

            for it in locations:
                if not it.longitude or not it.latitude:
                    # these ``it`` are brains, so anything which got lat/lng
                    # indexed can be used.
                    continue

                feature = {
                    "type": "Feature",
                    "id": it.UID,
                    "properties": {},
                    "geometry": {
                        "type": "Point",
                        "coordinates": [
                            it.longitude,
                            it.latitude,
                        ],
                    },
                }

                if count < limit:
                    props = IGeoJSONProperties(it.getObject(), None)
                    if getattr(props, "popup", None):
                        feature["properties"]["popup"] = props.popup
                    if getattr(props, "color", None):
                        feature["properties"]["color"] = props.color
                    if getattr(props, "extraClasses", None):
                        feature["properties"]["extraClasses"] = props.extraClasses

                features.append(feature)

            if not features:
                return

            geo_json = json.dumps({"type": "FeatureCollection", "features": features})
            return geo_json

        @property
        def locations(self):
            custom_query = {}  # Additional query to filter the collection

            collection = self.context
            if not collection:
                return None

            # Recursively transform all to unicode
            request_params = safe_decode(get_top_request(self.request).form or {})

            # Get all collection results with additional filter
            # defined by urlquery
            custom_query = base_query(request_params)
            custom_query = make_query(custom_query)
            return ICollectionish(collection).results(custom_query, request_params)


class BaseResetFilterView(BaseFilterView):
    @property
    def reset_url(self):
        return f"{self.collection.getURL()}?collectionfilter=1"

    @property
    def reset_class(self):
        return self.settings.css_class or ""



from datetime import datetime
from datetime import timedelta
from plone import api
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.textfield.value import RichTextValue
from plone.testing.zope import WSGI_SERVER_FIXTURE

import json
import os
import pytz


def _set_ajax_enabled(should_enable_ajax):
    pattern_options = api.portal.get_registry_record("plone.patternoptions")
    data = {"collectionfilter": str(json.dumps({"ajaxLoad": should_enable_ajax}))}
    pattern_options.update(data)
    api.portal.set_registry_record("plone.patternoptions", pattern_options)


class CollectiveCollectionFilterLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.app.mosaic

        self.loadZCML(package=plone.app.mosaic)
        import collective.geolocationbehavior

        self.loadZCML(package=collective.geolocationbehavior)
        import collective.collectionfilter

        self.loadZCML(package=collective.collectionfilter)

        import collective.collectionfilter.tests

        self.loadZCML(package=collective.collectionfilter.tests)

    def setUpPloneSite(self, portal):
        from plone.formwidget.geolocation.geolocation import Geolocation

        applyProfile(portal, "plone.app.mosaic:default")
        applyProfile(portal, "collective.geolocationbehavior:default")
        applyProfile(portal, "collective.collectionfilter:default")
        applyProfile(portal, "collective.collectionfilter.tests:testing")

        with api.env.adopt_roles(["Manager"]):
            portal.invokeFactory(
                "Collection",
                id="testcollection",
                title="Test Collection",
                query=[
                    {
                        "i": "portal_type",
                        "o": "plone.app.querystring.operation.selection.any",
                        "v": ["Document", "Event"],
                    }
                ],
            )
            now = datetime.now(pytz.UTC)

            portal.invokeFactory(
                "Event",
                id="testevent",
                title="Test Event",
                start=now + timedelta(days=1),
                end=now + timedelta(days=2),
                subject=["Süper", "Evänt"],
                exclude_from_nav=False,
            )
            portal.invokeFactory(
                "Document",
                id="testdoc",
                title="Test Document and Documént",
                text=RichTextValue(
                    "Ein heißes Test Dokument", "text/plain", "text/html"
                ),
                subject=["Süper", "Dokumänt"],
                exclude_from_nav=False,
            )
            portal.invokeFactory(
                "Document",
                id="testdoc2",
                title="Páge",
                text=RichTextValue(
                    "Ein heiBes Test Dokument", "text/plain", "text/html"
                ),
                subject=["Dokumänt"],
                exclude_from_nav=True,
            )
            doc = portal["testdoc"]
            doc.geolocation = Geolocation(47.4048832, 9.7587760701108)
            doc.reindexObject()

            portal.invokeFactory(
                "Folder",
                id="folder1",
                title="Folder with Contentelements",
                exclude_from_nav=False,
            )

            portal.invokeFactory(
                "Collection",
                id="mycollection",
                title="Test Multi Collection",
                query=[
                    {
                        "i": "portal_type",
                        "o": "plone.app.querystring.operation.selection.any",
                        "v": ["Document", "Event", "News Item"],
                    },
                    {
                        "i": "path",
                        "o": "plone.app.querystring.operation.string.absolutePath",
                        "v": "{}::-1".format(
                            portal.folder1.UID(),
                        ),
                    },
                ],
            )

            portal.folder1.invokeFactory(
                "Document",
                id="mydoc-red",
                title="Page Red",
                text=RichTextValue(
                    "Page 1 with Subject Red", "text/plain", "text/html"
                ),
                subject=["red"],
                exclude_from_nav=False,
            )

            portal.folder1.invokeFactory(
                "Document",
                id="mydoc-green",
                title="Page Green",
                text=RichTextValue(
                    "Page 2 with Subject Green", "text/plain", "text/html"
                ),
                subject=["green"],
                exclude_from_nav=True,
            )

            portal.folder1.invokeFactory(
                "Document",
                id="mydoc-blue",
                title="Page Blue",
                text=RichTextValue(
                    "Page 3 with Subject Blue", "text/plain", "text/html"
                ),
                subject=["blue"],
                exclude_from_nav=False,
            )

            portal.folder1.invokeFactory(
                "News Item",
                id="newsitem-red",
                title="News Item Red",
                text=RichTextValue(
                    "News Item 1 with Subject Red", "text/plain", "text/html"
                ),
                subject=["red"],
                exclude_from_nav=True,
            )

            portal.folder1.invokeFactory(
                "News Item",
                id="newsitem-blue",
                title="News Item Blue",
                text=RichTextValue(
                    "News Item 2 with Subject Blue", "text/plain", "text/html"
                ),
                subject=["blue"],
                exclude_from_nav=False,
            )

            portal.folder1.invokeFactory(
                "News Item",
                id="newsitem-green",
                title="News Item Green",
                text=RichTextValue(
                    "News Item 3 with Subject Green", "text/plain", "text/html"
                ),
                subject=["green"],
                exclude_from_nav=True,
            )


COLLECTIVE_COLLECTIONFILTER_FIXTURE = CollectiveCollectionFilterLayer()


COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_COLLECTIONFILTER_FIXTURE,),
    name="CollectiveCollectionFilterLayer:IntegrationTesting",
)

COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_COLLECTIONFILTER_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        WSGI_SERVER_FIXTURE,
    ),
    name="CollectiveCollectionFilterLayer:AcceptanceTesting",
)


class CollectiveCollectionFilterAjaxEnabledLayer(CollectiveCollectionFilterLayer):
    def setUpPloneSite(self, portal):
        _set_ajax_enabled(True)
        os.environ["ROBOT_AJAX_ENABLED"] = "True"
        super().setUpPloneSite(portal)

    def tearDownPloneSite(self, portal):
        super().tearDownPloneSite(portal)
        del os.environ["ROBOT_AJAX_ENABLED"]


AJAX_ENABLED_FIXTURE = CollectiveCollectionFilterAjaxEnabledLayer()
COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_ENABLED = FunctionalTesting(
    bases=(
        AJAX_ENABLED_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        WSGI_SERVER_FIXTURE,
    ),
    name="CollectiveCollectionFilterLayer:AcceptanceTestingPortlet_AjaxEnabled",
)


class CollectiveCollectionFilterAjaxDisabledLayer(CollectiveCollectionFilterLayer):
    def setUpPloneSite(self, portal):
        _set_ajax_enabled(False)
        super().setUpPloneSite(portal)


AJAX_DISABLED_FIXTURE = CollectiveCollectionFilterAjaxDisabledLayer()
COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_AJAX_DISABLED = FunctionalTesting(
    bases=(
        AJAX_DISABLED_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        WSGI_SERVER_FIXTURE,
    ),
    name="CollectiveCollectionFilterLayer:AcceptanceTestingPortlet_AjaxDisabled",
)


class CollectiveCollectionFilterTilesLayer(CollectiveCollectionFilterLayer):
    def setUpPloneSite(self, portal):
        os.environ["ROBOT_USE_TILES"] = "True"
        super().setUpPloneSite(portal)

    def tearDownPloneSite(self, portal):
        super().tearDownPloneSite(portal)
        del os.environ["ROBOT_USE_TILES"]


TILES_FIXTURE = CollectiveCollectionFilterTilesLayer()
COLLECTIVE_COLLECTIONFILTER_ACCEPTANCE_TESTING_TILES = FunctionalTesting(
    bases=(
        TILES_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        WSGI_SERVER_FIXTURE,
    ),
    name="CollectiveCollectionFilterLayer:AcceptanceTesting_Tiles",
)



"""Setup tests for this package."""

from collective.collectionfilter.filteritems import get_filter_items
from collective.collectionfilter.query import make_query
from collective.collectionfilter.testing import (  # noqa
    COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING,
)
from collective.collectionfilter.utils import safe_decode
from plone.app.contenttypes.interfaces import ICollection
from urllib.parse import parse_qs
from urllib.parse import urlparse

import unittest


def get_data_by_val(result, val):
    for r in result:
        if r["value"] == val:
            return r


def qs(result, index):
    url = get_data_by_val(result, index)["url"]
    _, _, _, _, query, _ = urlparse(url)
    result = parse_qs(query)
    del result["collectionfilter"]
    # Quick hack to get single values back from being lists
    result.update({k: v[0] for k, v in result.items() if len(v) == 1})
    return safe_decode(result)


class TestFilteritems(unittest.TestCase):
    layer = COLLECTIVE_COLLECTIONFILTER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.collection = self.portal["testcollection"]
        self.collection_uid = self.collection.UID()

    def test_filteritems(self):
        self.assertEqual(len(self.collection.results()), 6)

        result = get_filter_items(self.collection_uid, "Subject", cache_enabled=False)

        self.assertEqual(len(result), 7)
        self.assertEqual(get_data_by_val(result, "all")["count"], 6)
        self.assertEqual(get_data_by_val(result, "all")["selected"], True)
        self.assertEqual(get_data_by_val(result, "Süper")["count"], 2)
        self.assertEqual(get_data_by_val(result, "Evänt")["count"], 1)
        self.assertEqual(get_data_by_val(result, "Dokumänt")["count"], 2)

        result = get_filter_items(
            self.collection_uid,
            "Subject",
            request_params={"Subject": "Süper"},
            cache_enabled=False,
        )

        self.assertEqual(len(result), 7)
        self.assertEqual(get_data_by_val(result, "Süper")["selected"], True)

        result = get_filter_items(
            self.collection_uid,
            "Subject",
            request_params={"Subject": "Dokumänt"},
            cache_enabled=False,
        )

        self.assertEqual(len(result), 7)
        self.assertEqual(get_data_by_val(result, "Dokumänt")["selected"], True)

        # test narrowed down results
        narrowed_down_result = get_filter_items(
            self.collection_uid,
            "Subject",
            request_params={"Subject": "Dokumänt"},
            narrow_down=True,
            show_count=True,
            cache_enabled=False,
        )

        self.assertEqual(
            len(narrowed_down_result), 3, msg="narrowed result length should be 3"
        )
        self.assertEqual(
            get_data_by_val(narrowed_down_result, "Dokumänt")["selected"],
            True,  # noqa
            msg="Test that 'Dokumänt' is selected, matching the query",
        )
        self.assertEqual(
            get_data_by_val(narrowed_down_result, "all")["count"],
            6,
            msg="Test that there are 3 results if unselected",
        )

    def test_portal_type_filter(self):
        self.assertEqual(len(self.collection.results()), 6)

        result = get_filter_items(
            self.collection_uid, "portal_type", cache_enabled=False
        )

        self.assertEqual(len(result), 3)
        self.assertEqual(get_data_by_val(result, "all")["count"], 6)
        self.assertEqual(get_data_by_val(result, "all")["selected"], True)
        self.assertEqual(get_data_by_val(result, "Event")["count"], 1)
        self.assertEqual(get_data_by_val(result, "Document")["count"], 5)

        result = get_filter_items(
            self.collection_uid,
            "portal_type",
            request_params={"portal_type": "Event"},
            cache_enabled=False,
        )

        self.assertEqual(len(result), 3)
        self.assertEqual(get_data_by_val(result, "Event")["selected"], True)

        # test narrowed down results
        result = get_filter_items(
            self.collection_uid,
            "portal_type",
            request_params={"portal_type": "Event"},
            narrow_down=True,
            show_count=True,
            cache_enabled=False,
        )

        self.assertEqual(len(result), 2)
        self.assertEqual(
            get_data_by_val(result, "all")["count"],
            6,
            msg="Test that the number of results if unselected is 3",
        )

        self.assertEqual(
            get_data_by_val(result, "Event")["selected"],
            True,
            msg="Test that Event portal_type is selected matching the query",
        )

        # test operators option on FieldIndex
        result = get_filter_items(
            self.collection_uid,
            "portal_type",
            request_params={
                "portal_type": ["Event", "Document"],
                "portal_type_op": "or",
            },
            filter_type="or",
            cache_enabled=False,
        )

        self.assertEqual(len(result), 3)
        self.assertEqual(get_data_by_val(result, "Event")["selected"], True)
        self.assertEqual(get_data_by_val(result, "Document")["selected"], True)  # noqa

        # and operator is ignored (same result as above)
        result = get_filter_items(
            self.collection_uid,
            "portal_type",
            request_params={
                "portal_type": ["Event", "Document"],
                "portal_type_op": "and",
            },
            filter_type="and",
            cache_enabled=False,
        )

        self.assertEqual(len(result), 3)

    def test_and_filter_type(self):
        self.assertEqual(len(self.collection.results()), 6)

        result = get_filter_items(self.collection_uid, "Subject", cache_enabled=False)

        self.assertEqual(len(result), 7)
        self.assertEqual(get_data_by_val(result, "all")["count"], 6)
        self.assertEqual(get_data_by_val(result, "all")["selected"], True)
        self.assertEqual(get_data_by_val(result, "Süper")["count"], 2)
        self.assertEqual(get_data_by_val(result, "Evänt")["count"], 1)
        self.assertEqual(get_data_by_val(result, "Dokumänt")["count"], 2)

        # Test url
        self.assertEqual(qs(result, "Süper"), {"Subject": "Süper"})

        catalog_results = ICollection(self.collection).results(
            batch=False, brains=True, custom_query=make_query(qs(result, "Süper"))
        )
        self.assertEqual(len(catalog_results), 2)

        result = get_filter_items(
            self.collection_uid,
            "Subject",
            request_params={"Subject": "Süper"},
            filter_type="and",
            cache_enabled=False,
        )

        self.assertEqual(len(result), 7)
        self.assertEqual(get_data_by_val(result, "all")["count"], 6)

        # TODO: I'm not sure these counts are correct. It should represent how many results you will get if you click
        # so should be smaller than this but I guess you need to turn on narrow down for that?
        self.assertEqual(get_data_by_val(result, "Süper")["count"], 2)
        self.assertEqual(get_data_by_val(result, "Evänt")["count"], 1)
        self.assertEqual(get_data_by_val(result, "Dokumänt")["count"], 2)

        self.assertEqual(get_data_by_val(result, "Süper")["selected"], True)

        self.assertEqual(qs(result, "Süper"), {})
        self.assertEqual(
            qs(result, "Dokumänt"),
            {"Subject_op": "and", "Subject": ["Süper", "Dokumänt"]},
        )
        self.assertEqual(
            qs(result, "Evänt"), {"Subject_op": "and", "Subject": ["Süper", "Evänt"]}
        )

        # Narrow down by 2

        catalog_results = ICollection(self.collection).results(
            batch=False, brains=True, custom_query=make_query(qs(result, "Dokumänt"))
        )
        self.assertEqual(len(catalog_results), 1)

        result = get_filter_items(
            self.collection_uid,
            "Subject",
            request_params={"Subject": ["Süper", "Dokumänt"]},
            filter_type="and",
            cache_enabled=False,
        )

        self.assertEqual(len(result), 7)
        self.assertEqual(get_data_by_val(result, "all")["count"], 6)
        self.assertEqual(get_data_by_val(result, "Süper")["count"], 2)

        self.assertEqual(get_data_by_val(result, "Evänt")["count"], 1)
        self.assertEqual(get_data_by_val(result, "Dokumänt")["count"], 2)

        self.assertEqual(get_data_by_val(result, "Süper")["selected"], True)
        self.assertEqual(get_data_by_val(result, "Dokumänt")["selected"], True)

        self.assertEqual(qs(result, "Süper"), {"Subject": "Dokumänt"})
        self.assertEqual(qs(result, "Dokumänt"), {"Subject": "Süper"})
        self.assertEqual(
            qs(result, "Evänt"),
            {"Subject": ["Süper", "Dokumänt", "Evänt"], "Subject_op": "and"},
        )

        # Clicking on Event we should get 0 results as none will be in common
        catalog_results = ICollection(self.collection).results(
            batch=False, brains=True, custom_query=make_query(qs(result, "Evänt"))
        )
        self.assertEqual(len(catalog_results), 0)

    def test_boolean_filter(self):
        """Validate boolean fields are shown with all values."""
        self.assertEqual(len(self.collection.results()), 6)

        result = get_filter_items(
            self.collection_uid, "exclude_from_nav", cache_enabled=False
        )

        self.assertEqual(len(result), 3)
        self.assertEqual(get_data_by_val(result, "all")["count"], 6)
        self.assertEqual(get_data_by_val(result, "all")["selected"], True)
        self.assertEqual(get_data_by_val(result, True)["count"], 2)
        self.assertEqual(get_data_by_val(result, False)["count"], 4)

        # test narrowed down results
        narrowed_down_result = get_filter_items(
            self.collection_uid,
            "exclude_from_nav",
            request_params={"exclude_from_nav": True},
            narrow_down=True,
            show_count=True,
            cache_enabled=False,
        )

        self.assertEqual(
            len(narrowed_down_result), 2, msg="narrowed result length should be 2"
        )
        self.assertEqual(
            get_data_by_val(narrowed_down_result, True)["selected"],
            True,  # noqa
            msg="Test that 'Yes' is selected, matching the query",
        )
        self.assertEqual(
            get_data_by_val(narrowed_down_result, "all")["count"],
            6,
            msg="Test that there are 3 results if unselected",
        )

        # test narrowed down results
        narrowed_down_result = get_filter_items(
            self.collection_uid,
            "exclude_from_nav",
            request_params={"exclude_from_nav": False},
            narrow_down=True,
            show_count=True,
            cache_enabled=False,
        )

        self.assertEqual(
            len(narrowed_down_result), 2, msg="narrowed result length should be 2"
        )
        self.assertEqual(
            get_data_by_val(narrowed_down_result, False)["selected"],
            True,  # noqa
            msg="Test that 'No' is selected, matching the query",
        )
        self.assertEqual(
            get_data_by_val(narrowed_down_result, "all")["count"],
            6,
            msg="Test that there are 3 results if unselected",
        )



from setuptools import find_packages
from setuptools import setup

import os


version = "5.2.1"


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


setup(
    name="collective.collectionfilter",
    version=version,
    description="Plone addon for filtering collection results.",
    long_description="{}\n\n{}".format(
        read("README.rst"),
        read("CHANGES.rst"),
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Addon",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="plone collection filter faceted tagcloud tags",
    author="Johannes Raggam",
    author_email="thetetet@gmail.com",
    url="http://github.com/collective/collective.collectionfilter",
    license="GPL",
    namespace_packages=[
        "collective",
    ],
    python_requires=">=3.8",
    packages=find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "setuptools",
        "plone.api >= 2.0.0",
        "plone.app.contenttypes",
        "plone.app.event",
        "plone.app.querystring",
        "plone.app.uuid",
        "plone.app.vocabularies",
        "plone.app.z3cform",
        "plone.autoform",
        "plone.base",
        "plone.dexterity",
        "plone.i18n",
        "plone.memoize",
        "plone.portlets",
        "plone.supermodel",
        "plone.uuid",
        "z3c.form",
        "Products.CMFCore",
        "Products.CMFPlone >= 6.0",
    ],
    extras_require={
        "mosaic": [
            "plone.app.mosaic >= 3.0.0",
        ],
        "geolocation": [
            # support for latitude/longitude catalog index
            "collective.geolocationbehavior >= 1.7.2",
            # refactored map configuration
            "plone.formwidget.geolocation >= 3.0.0",
        ],
        "test": [
            "plone.app.mosaic",
            "plone.browserlayer",
            "plone.testing",
            "robotsuite",
            "collective.geolocationbehavior",
            "plone.app.testing[robot]",
            "plone.app.robotframework",
            "plone.app.contenttypes",
            "plone.app.portlets",
            "plone.app.textfield",
            "plone.tiles",
        ],
    },
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



from plone import api
from Products.CMFCore.interfaces import IFolderish
from Products.CMFPlone.utils import safe_unicode


try:
    from plone.app.blocks.layoutbehavior import ILayoutAware
except ImportError:
    ILayoutAware = None


def target_collection_base_path(context):
    for potential_context in context.aq_chain:
        if IFolderish.providedBy(potential_context):
            # Actually makes no sense to pick the collection path as it means you have to go up
            # to pick the collection. Instead the collection should be the default
            #    or
            # ISyndicatableCollection.providedBy(potential_context
            #                                   )
            context = potential_context
            break
    return "/".join(context.getPhysicalPath())


def target_collection_types(context):
    if ILayoutAware:
        default = ["Collection", "Page", "Folder"]
    else:
        default = ["Collection"]
    return api.portal.get_registry_record(
        "collective.collectionfilter.target_collection_types", default=default
    )


def safe_decode(val):
    """Safely create unicode values."""
    if isinstance(val, dict):
        return {
            safe_decode(k): safe_decode(v) for k, v in val.items() if v is not None
        }  # noqa
    if isinstance(val, list):
        return [safe_decode(it) for it in val]
    if isinstance(val, tuple):
        return (safe_decode(it) for it in val)
    if val:
        return safe_unicode(val)
    return val


def safe_encode(val):
    """Safely encode a value to utf-8."""
    if isinstance(val, dict):
        return {
            safe_encode(k): safe_encode(v) for k, v in val.items() if v is not None
        }  # noqa
    if isinstance(val, list):
        return [safe_encode(it) for it in val]
    if isinstance(val, tuple):
        return (safe_encode(it) for it in val)
    if isinstance(val, str):
        return safe_unicode(val).encode("utf-8")
    return val


def safe_iterable(value):
    if value is None:
        return []
    if isinstance(value, str):
        # do not expand a string to a list of chars
        return [value]
    try:
        return list(value)
    except TypeError:
        # int and other stuff
        return [value]


def clean_query(urlquery, ignores):
    """remove all to-be-ignored request parameters."""
    cleaned_query = urlquery.copy()
    for it in ignores:
        # Remove problematic url parameters
        if it in cleaned_query:
            # full match
            del cleaned_query[it]
        else:
            # enhanced partial check
            for urlkey in cleaned_query:
                # also delete prefixed (like with tile id) or
                # postfixed (like :int) keys
                if it in urlkey:
                    del cleaned_query[urlkey]
                    break
    return cleaned_query


def base_query(request_params={}, extra_ignores=[]):
    """Construct base url query."""

    # Defaults
    request_params = request_params or {}
    extra_ignores = extra_ignores or []

    # These request params should be ignored.
    ignore_params = [
        "b_start",
        "b_size",
        "batch",
        "limit",
        "portlethash",
    ] + extra_ignores
    urlquery = clean_query(request_params, ignore_params)
    urlquery.update({"collectionfilter": "1"})  # marker
    return urlquery


def get_top_request(request):
    """Get highest request from a subrequest."""

    def _top_request(req):
        parent_request = req.get("PARENT_REQUEST", None)
        return _top_request(parent_request) if parent_request else req

    return _top_request(request)



from collective.collectionfilter.baseviews import BaseSearchView
from collective.collectionfilter.interfaces import ICollectionSearchSchema
from collective.collectionfilter.tiles import BaseFilterTile
from plone.supermodel.model import Schema
from zope.interface import implementer


class ISearchTile(Schema, ICollectionSearchSchema):
    pass


@implementer(ISearchTile)
class SearchTile(BaseFilterTile, BaseSearchView):
    """Search Tile"""



from collective.collectionfilter import _
from collective.collectionfilter.baseviews import BaseSortOnView
from collective.collectionfilter.interfaces import (  # noqa
    ICollectionFilterResultListSort,
)
from collective.collectionfilter.portlets import BasePortletRenderer
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implementer


class ICollectionFilterSortOnPortlet(
    ICollectionFilterResultListSort, IPortletDataProvider
):  # noqa
    """Portlet interface based on ICollectionFilterSchema"""


@implementer(ICollectionFilterSortOnPortlet)
class Assignment(base.Assignment):
    header = ""
    target_collection = None
    view_name = None
    sort_on = ""
    input_type = "links"
    content_selector = "#content-core"

    def __init__(
        self,
        header="",
        target_collection=None,
        sort_on="",
        input_type="links",
        view_name=None,
        content_selector="#content-core",
    ):
        self.header = header
        self.target_collection = target_collection
        self.sort_on = sort_on
        self.input_type = input_type
        self.view_name = view_name
        self.content_selector = content_selector

    @property
    def portlet_id(self):
        """Return the portlet assignment's unique object id."""
        return id(self)

    @property
    def title(self):
        if self.header:
            return self.header
        else:
            return _("Collection Result Sorting")


class Renderer(BasePortletRenderer, BaseSortOnView):
    render = ViewPageTemplateFile("sorting.pt")


class AddForm(base.AddForm):
    schema = ICollectionFilterSortOnPortlet
    label = _("Add Collection Result Listing Sort Portlet")
    description = _("This portlet shows sorting options for the result listing.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    schema = ICollectionFilterSortOnPortlet
    label = _("Edit Collection Result Listing Sort Portlet")
    description = _("This portlet shows sorting options for the result listing.")
